﻿# Use this script to log on to - and check for - updated, compressed database file on SIL SFTP server.
# File will only get downloaded from the server if it's more recent than the one - or missing - in the client download directory.
# Last edited 2022-09-27.

# IMPORTANT
# Read more on how to get access at our official web page: https://www.inera.se/sil

# Server options
$server = "sftp.silinfo.se"
$server_port = "22"
$server_fingerprint = "ssh-rsa 4096 b3:d4:e6:76:0d:86:41:48:5d:8e:ba:5f:b7:c1:c5:c5"
$server_path = "/data/70/SilDB70.zip"

# Client options (adjustments required)
$user = "YOUR_USERNAME"
$client_private_key_file = "id_rsa.ppk"
$client_path = "downloaded_files\SilDB70.zip"

try
{
    # Load WinSCP .NET assembly
    Add-Type -Path "WinSCP.dll"

    # Session.FileTransferProgress event handler
 
    function FileTransferProgress
    {
        param($e)
 
        # New line for every new file
        if (($script:lastFileName -ne $Null) -and
            ($script:lastFileName -ne $e.FileName))
        {
            Write-Host
        }
 
        # Print transfer progress
        Write-Host -NoNewline ("`rDownloading... {0} ({1:P0})" -f $e.FileName, $e.FileProgress)
 
        # Remember a name of the last file reported
        $script:lastFileName = $e.FileName
    }

    # Main script
 
    $script:lastFileName = $Null
 
    # Setup session options
    $sessionOptions = New-Object WinSCP.SessionOptions
    $sessionOptions.Protocol = [WinSCP.Protocol]::Sftp
    $sessionOptions.HostName = "$server"
    $sessionOptions.PortNumber = "$server_port"
    $sessionOptions.UserName = "$user"
    $sessionOptions.SshHostKeyFingerprint = "$server_fingerprint"
    $sessionOptions.SshPrivateKeyPath = "$client_private_key_file"
 
    $session = New-Object WinSCP.Session
 
    try
    {
        # Will continuously report progress of transfer
        $session.add_FileTransferProgress( { FileTransferProgress($_) } )

        # Connect
        $session.Open($sessionOptions)
 
        if ($session.FileExists($server_path))
        {
            if (!(Test-Path $client_path))
            {
                Write-Host (
                    "{0} Remote file ({1}) exists, local version ({2}) does not." -f
                    (get-date), $server_path, $client_path)
                Write-Host ("{0} Starting download." -f (get-date))
                $download = $True
            }
            else
            {
                $remoteWriteTime = $session.GetFileInfo($server_path).LastWriteTime
                $localWriteTime = (Get-Item $client_path).LastWriteTime
 
                if ($remoteWriteTime -gt $localWriteTime)
                {
                    Write-Host (
                        ("{0} Remote file ({1}) as well as local version ({2}) exist, but remote file is newer ({3}) than the local version ({4}).") -f
                        (get-date), $server_path, $client_path, $remoteWriteTime, $localWriteTime)
                    Write-Host ("{0} Updating local version of the file..." -f (get-date))
                    $download = $True
                }
                else
                {
                    Write-Host (
                        ("{0} Remote file ({1}) as well as local version ({2}) exist, but remote file is NOT newer ({3}) than the local version ({4}).") -f
                        (get-date), $server_path, $client_path, $remoteWriteTime, $localWriteTime)
                    Write-Host ("{0} Will NOT attempt to download." -f (get-date))
                    $download = $False
                }
            }
 
            if ($download)
            {
                # Download the file and throw on any error
                $session.GetFiles($server_path, $client_path).Check()
 
                Write-Host ("`r{0} Download complete." -f (get-date))
            }
        }
        else
        {
            Write-Host ("{0} Error: Remote file ({1}) does not exist." -f (get-date), $server_path)
            Write-Host ("{0} Unable to download." -f (get-date))
        }
    }
    finally
    {
        # Terminate line after the last file (if any)
        if ($script:lastFileName -ne $Null)
        {
            Write-Host
        }

        # Disconnect, clean up
        $session.Dispose()
    }
 
    exit 0
}
catch [Exception]
{
    Write-Host ("{0} {1}" -f (get-date), $_.Exception.Message)
    exit 1
}